import { EventEmitter, Directive, IterableDiffers, Input, Output, Component, Optional, ViewChild, NgModule } from '@angular/core';
import { includes, orderBy, slice, min } from 'lodash';
import { ReplaySubject } from 'rxjs';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * Generated from: lib/DataTable.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DataTable {
    /**
     * @param {?} differs
     */
    constructor(differs) {
        this.differs = differs;
        this.inputData = [];
        this.sortBy = "";
        this.sortOrder = "asc";
        this.sortByChange = new EventEmitter();
        this.sortOrderChange = new EventEmitter();
        this.rowsOnPage = 1000;
        this.activePage = 1;
        this.mustRecalculateData = false;
        this.onSortChange = new ReplaySubject(1);
        this.onPageChange = new EventEmitter();
        this.diff = differs.find([]).create(null);
    }
    /**
     * @return {?}
     */
    getSort() {
        return { sortBy: this.sortBy, sortOrder: this.sortOrder };
    }
    /**
     * @param {?} sortBy
     * @param {?} sortOrder
     * @return {?}
     */
    setSort(sortBy, sortOrder) {
        if (this.sortBy !== sortBy || this.sortOrder !== sortOrder) {
            this.sortBy = sortBy;
            this.sortOrder = includes(["asc", "desc"], sortOrder) ? sortOrder : "asc";
            this.mustRecalculateData = true;
            this.onSortChange.next({ sortBy: sortBy, sortOrder: sortOrder });
            this.sortByChange.emit(this.sortBy);
            this.sortOrderChange.emit(this.sortOrder);
        }
    }
    /**
     * @return {?}
     */
    getPage() {
        return { activePage: this.activePage, rowsOnPage: this.rowsOnPage, dataLength: this.inputData.length };
    }
    /**
     * @param {?} activePage
     * @param {?} rowsOnPage
     * @return {?}
     */
    setPage(activePage, rowsOnPage) {
        if (this.rowsOnPage !== rowsOnPage || this.activePage !== activePage) {
            this.activePage = this.activePage !== activePage ? activePage : this.calculateNewActivePage(this.rowsOnPage, rowsOnPage);
            this.rowsOnPage = rowsOnPage;
            this.mustRecalculateData = true;
            this.onPageChange.emit({
                activePage: this.activePage,
                rowsOnPage: this.rowsOnPage,
                dataLength: this.inputData ? this.inputData.length : 0
            });
        }
    }
    /**
     * @private
     * @param {?} previousRowsOnPage
     * @param {?} currentRowsOnPage
     * @return {?}
     */
    calculateNewActivePage(previousRowsOnPage, currentRowsOnPage) {
        /** @type {?} */
        const firstRowOnPage = (this.activePage - 1) * previousRowsOnPage + 1;
        return Math.ceil(firstRowOnPage / currentRowsOnPage);
    }
    /**
     * @private
     * @return {?}
     */
    recalculatePage() {
        /** @type {?} */
        let lastPage = Math.ceil(this.inputData.length / this.rowsOnPage);
        this.activePage = lastPage < this.activePage ? lastPage : this.activePage;
        this.activePage = this.activePage || 1;
        this.onPageChange.emit({
            activePage: this.activePage,
            rowsOnPage: this.rowsOnPage,
            dataLength: this.inputData.length
        });
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes["rowsOnPage"]) {
            this.rowsOnPage = changes["rowsOnPage"].previousValue;
            this.setPage(this.activePage, changes["rowsOnPage"].currentValue);
            this.mustRecalculateData = true;
        }
        if (changes["sortBy"] || changes["sortOrder"]) {
            if (!includes(["asc", "desc"], this.sortOrder)) {
                console.warn("angular2-datatable: value for input mfSortOrder must be one of ['asc', 'desc'], but is:", this.sortOrder);
                this.sortOrder = "asc";
            }
            if (this.sortBy) {
                this.onSortChange.next({ sortBy: this.sortBy, sortOrder: this.sortOrder });
            }
            this.mustRecalculateData = true;
        }
        if (changes["inputData"]) {
            this.inputData = changes["inputData"].currentValue || [];
            this.recalculatePage();
            this.mustRecalculateData = true;
        }
    }
    /**
     * @return {?}
     */
    ngDoCheck() {
        /** @type {?} */
        let changes = this.diff.diff(this.inputData);
        if (changes) {
            this.recalculatePage();
            this.mustRecalculateData = true;
        }
        if (this.mustRecalculateData) {
            this.fillData();
            this.mustRecalculateData = false;
        }
    }
    /**
     * @private
     * @return {?}
     */
    fillData() {
        /** @type {?} */
        let offset = (this.activePage - 1) * this.rowsOnPage;
        /** @type {?} */
        let data = this.inputData;
        /** @type {?} */
        const sortBy = this.sortBy;
        if (typeof sortBy === 'string' || sortBy instanceof String) {
            data = orderBy(data, this.caseInsensitiveIteratee((/** @type {?} */ (sortBy))), [this.sortOrder]);
        }
        else {
            data = orderBy(data, sortBy, [this.sortOrder]);
        }
        data = slice(data, offset, offset + this.rowsOnPage);
        this.data = data;
    }
    /**
     * @private
     * @param {?} sortBy
     * @return {?}
     */
    caseInsensitiveIteratee(sortBy) {
        return (/**
         * @param {?} row
         * @return {?}
         */
        (row) => {
            /** @type {?} */
            let value = row;
            for (let sortByProperty of sortBy.split('.')) {
                if (value) {
                    value = value[sortByProperty];
                }
            }
            if (value && typeof value === 'string' || value instanceof String) {
                return value.toLowerCase();
            }
            return value;
        });
    }
}
DataTable.decorators = [
    { type: Directive, args: [{
                selector: 'table[mfData]',
                exportAs: 'mfDataTable'
            },] }
];
/** @nocollapse */
DataTable.ctorParameters = () => [
    { type: IterableDiffers }
];
DataTable.propDecorators = {
    inputData: [{ type: Input, args: ["mfData",] }],
    sortBy: [{ type: Input, args: ["mfSortBy",] }],
    sortOrder: [{ type: Input, args: ["mfSortOrder",] }],
    sortByChange: [{ type: Output, args: ["mfSortByChange",] }],
    sortOrderChange: [{ type: Output, args: ["mfSortOrderChange",] }],
    rowsOnPage: [{ type: Input, args: ["mfRowsOnPage",] }],
    activePage: [{ type: Input, args: ["mfActivePage",] }]
};
if (false) {
    /**
     * @type {?}
     * @private
     */
    DataTable.prototype.diff;
    /** @type {?} */
    DataTable.prototype.inputData;
    /** @type {?} */
    DataTable.prototype.sortBy;
    /** @type {?} */
    DataTable.prototype.sortOrder;
    /** @type {?} */
    DataTable.prototype.sortByChange;
    /** @type {?} */
    DataTable.prototype.sortOrderChange;
    /** @type {?} */
    DataTable.prototype.rowsOnPage;
    /** @type {?} */
    DataTable.prototype.activePage;
    /**
     * @type {?}
     * @private
     */
    DataTable.prototype.mustRecalculateData;
    /** @type {?} */
    DataTable.prototype.data;
    /** @type {?} */
    DataTable.prototype.onSortChange;
    /** @type {?} */
    DataTable.prototype.onPageChange;
    /**
     * @type {?}
     * @private
     */
    DataTable.prototype.differs;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/BootstrapPaginator.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BootstrapPaginator {
    constructor() {
        this.rowsOnPageSet = [];
        this.minRowsOnPage = 0;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.rowsOnPageSet) {
            this.minRowsOnPage = min(this.rowsOnPageSet);
        }
    }
}
BootstrapPaginator.decorators = [
    { type: Component, args: [{
                selector: "mfBootstrapPaginator",
                template: `
    <mfPaginator #p [mfTable]="mfTable">
        <ul class="pagination" *ngIf="p.dataLength > p.rowsOnPage">
            <li class="page-item" [class.disabled]="p.activePage <= 1" (click)="p.setPage(1)">
                <a class="page-link" style="cursor: pointer">&laquo;</a>
            </li>
            <li class="page-item" *ngIf="p.activePage > 4 && p.activePage + 1 > p.lastPage" (click)="p.setPage(p.activePage - 4)">
                <a class="page-link" style="cursor: pointer">{{p.activePage-4}}</a>
            </li>
            <li class="page-item" *ngIf="p.activePage > 3 && p.activePage + 2 > p.lastPage" (click)="p.setPage(p.activePage - 3)">
                <a class="page-link" style="cursor: pointer">{{p.activePage-3}}</a>
            </li>
            <li class="page-item" *ngIf="p.activePage > 2" (click)="p.setPage(p.activePage - 2)">
                <a class="page-link" style="cursor: pointer">{{p.activePage-2}}</a>
            </li>
            <li class="page-item" *ngIf="p.activePage > 1" (click)="p.setPage(p.activePage - 1)">
                <a class="page-link" style="cursor: pointer">{{p.activePage-1}}</a>
            </li>
            <li class="page-item active">
                <a class="page-link" style="cursor: pointer">{{p.activePage}}</a>
            </li>
            <li class="page-item" *ngIf="p.activePage + 1 <= p.lastPage" (click)="p.setPage(p.activePage + 1)">
                <a class="page-link" style="cursor: pointer">{{p.activePage+1}}</a>
            </li>
            <li class="page-item" *ngIf="p.activePage + 2 <= p.lastPage" (click)="p.setPage(p.activePage + 2)">
                <a class="page-link" style="cursor: pointer">{{p.activePage+2}}</a>
            </li>
            <li class="page-item" *ngIf="p.activePage + 3 <= p.lastPage && p.activePage < 3" (click)="p.setPage(p.activePage + 3)">
                <a class="page-link" style="cursor: pointer">{{p.activePage+3}}</a>
            </li>
            <li class="page-item" *ngIf="p.activePage + 4 <= p.lastPage && p.activePage < 2" (click)="p.setPage(p.activePage + 4)">
                <a class="page-link" style="cursor: pointer">{{p.activePage+4}}</a>
            </li>
            <li class="page-item" [class.disabled]="p.activePage >= p.lastPage" (click)="p.setPage(p.lastPage)">
                <a class="page-link" style="cursor: pointer">&raquo;</a>
            </li>
        </ul>
        <ul class="pagination pull-right float-sm-right" *ngIf="p.dataLength > minRowsOnPage">
            <li class="page-item" *ngFor="let rows of rowsOnPageSet" [class.active]="p.rowsOnPage===rows" (click)="p.setRowsOnPage(rows)">
                <a class="page-link" style="cursor: pointer">{{rows}}</a>
            </li>
        </ul>
    </mfPaginator>
    `
            }] }
];
BootstrapPaginator.propDecorators = {
    rowsOnPageSet: [{ type: Input, args: ["rowsOnPageSet",] }],
    mfTable: [{ type: Input, args: ["mfTable",] }]
};
if (false) {
    /** @type {?} */
    BootstrapPaginator.prototype.rowsOnPageSet;
    /** @type {?} */
    BootstrapPaginator.prototype.mfTable;
    /** @type {?} */
    BootstrapPaginator.prototype.minRowsOnPage;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/DataTableBody.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template R
 */
class DataTableBody {
    /**
     * @param {?} mfTable
     */
    constructor(mfTable) {
        this.mfTable = mfTable;
    }
}
DataTableBody.decorators = [
    { type: Component, args: [{
                selector: "[mfBody]",
                template: `<tr *ngFor="let row of mfTable.data">
        <td *ngFor="let column of columns"
            [attr.title]="column.valueTooltip(row) || null"
            [attr.class]="column.classname || null"
            [innerHtml]="column.valueHtml(row)"></td>
    </tr>`
            }] }
];
/** @nocollapse */
DataTableBody.ctorParameters = () => [
    { type: DataTable }
];
DataTableBody.propDecorators = {
    columns: [{ type: Input, args: ["columns",] }]
};
if (false) {
    /** @type {?} */
    DataTableBody.prototype.columns;
    /** @type {?} */
    DataTableBody.prototype.mfTable;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/DataTableHead.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template R
 */
class DataTableHead {
}
DataTableHead.decorators = [
    { type: Component, args: [{
                selector: "[mfHead]",
                template: `<tr>
        <th *ngFor="let column of columns"><mfDefaultSorter by="{{column.field}}" title="{{column.tooltip}}">{{column.name}}</mfDefaultSorter></th>
    </tr>`
            }] }
];
DataTableHead.propDecorators = {
    columns: [{ type: Input, args: ["columns",] }]
};
if (false) {
    /** @type {?} */
    DataTableHead.prototype.columns;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/DefaultSorter.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DefaultSorter {
    /**
     * @param {?} mfTable
     */
    constructor(mfTable) {
        this.mfTable = mfTable;
        this.isSortedByMeAsc = false;
        this.isSortedByMeDesc = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.mfTable.onSortChange.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            this.isSortedByMeAsc = (event.sortBy == this.sortBy && event.sortOrder == "asc");
            this.isSortedByMeDesc = (event.sortBy == this.sortBy && event.sortOrder == "desc");
        }));
    }
    /**
     * @return {?}
     */
    sort() {
        if (this.isSortedByMeAsc) {
            this.mfTable.setSort(this.sortBy, "desc");
        }
        else {
            this.mfTable.setSort(this.sortBy, "asc");
        }
    }
}
DefaultSorter.decorators = [
    { type: Component, args: [{
                selector: "mfDefaultSorter",
                template: `
        <a style="cursor: pointer" (click)="sort()" class="text-nowrap">
            <ng-content></ng-content>
            <span *ngIf="isSortedByMeAsc" class="glyphicon glyphicon-triangle-top" aria-hidden="true"></span>
            <span *ngIf="isSortedByMeDesc" class="glyphicon glyphicon-triangle-bottom" aria-hidden="true"></span>
        </a>`
            }] }
];
/** @nocollapse */
DefaultSorter.ctorParameters = () => [
    { type: DataTable }
];
DefaultSorter.propDecorators = {
    sortBy: [{ type: Input, args: ["by",] }]
};
if (false) {
    /** @type {?} */
    DefaultSorter.prototype.sortBy;
    /** @type {?} */
    DefaultSorter.prototype.isSortedByMeAsc;
    /** @type {?} */
    DefaultSorter.prototype.isSortedByMeDesc;
    /**
     * @type {?}
     * @private
     */
    DefaultSorter.prototype.mfTable;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/Paginator.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Paginator {
    /**
     * @param {?} injectMfTable
     */
    constructor(injectMfTable) {
        this.injectMfTable = injectMfTable;
        this.dataLength = 0;
        this.onPageChangeSubscriber = (/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            this.activePage = event.activePage;
            this.rowsOnPage = event.rowsOnPage;
            this.dataLength = event.dataLength;
            this.lastPage = Math.ceil(this.dataLength / this.rowsOnPage);
        });
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.mfTable = this.inputMfTable || this.injectMfTable;
        this.onPageChangeSubscriber(this.mfTable.getPage());
        this.mfTable.onPageChange.subscribe(this.onPageChangeSubscriber);
    }
    /**
     * @param {?} pageNumber
     * @return {?}
     */
    setPage(pageNumber) {
        this.mfTable.setPage(pageNumber, this.rowsOnPage);
    }
    /**
     * @param {?} rowsOnPage
     * @return {?}
     */
    setRowsOnPage(rowsOnPage) {
        this.mfTable.setPage(this.activePage, rowsOnPage);
    }
}
Paginator.decorators = [
    { type: Component, args: [{
                selector: "mfPaginator",
                template: `<ng-content></ng-content>`
            }] }
];
/** @nocollapse */
Paginator.ctorParameters = () => [
    { type: DataTable, decorators: [{ type: Optional }] }
];
Paginator.propDecorators = {
    inputMfTable: [{ type: Input, args: ["mfTable",] }]
};
if (false) {
    /** @type {?} */
    Paginator.prototype.inputMfTable;
    /**
     * @type {?}
     * @private
     */
    Paginator.prototype.mfTable;
    /** @type {?} */
    Paginator.prototype.activePage;
    /** @type {?} */
    Paginator.prototype.rowsOnPage;
    /** @type {?} */
    Paginator.prototype.dataLength;
    /** @type {?} */
    Paginator.prototype.lastPage;
    /**
     * @type {?}
     * @private
     */
    Paginator.prototype.onPageChangeSubscriber;
    /**
     * @type {?}
     * @private
     */
    Paginator.prototype.injectMfTable;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/TableController.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TableController {
    constructor() {
        this.placeholder = 'search term';
        this.anywhere = false;
        this.searchValue = "";
        this.rowsOnPageSet = [20, 30, 50];
        this.minRowsOnPage = 0;
    }
    /**
     * @return {?}
     */
    pages() {
        /** @type {?} */
        const result = [];
        /** @type {?} */
        const totalPages = this.paginator.lastPage;
        addPage(1, totalPages, result);
        addPage((this.paginator.activePage + 1) / 2, totalPages, result);
        addPage(this.paginator.activePage - 1, totalPages, result);
        addPage((this.paginator.activePage + totalPages) / 2, totalPages, result);
        addPage(this.paginator.activePage, totalPages, result);
        addPage(this.paginator.activePage + 1, totalPages, result);
        addPage(totalPages, totalPages, result);
        if (result.length < 7) {
            addPage((this.paginator.activePage + totalPages / 10), totalPages, result);
            addPage(this.paginator.activePage - totalPages / 10, totalPages, result);
        }
        result.sort((/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        (a, b) => a - b));
        return result;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    search(event) {
        /** @type {?} */
        const searchTerm = event.srcElement.value.toLowerCase();
        /** @type {?} */
        let index = 0;
        for (const row of this.getSortedData()) {
            /** @type {?} */
            const v = row[this.searchColumn].toString();
            if ((!this.anywhere && v.toLowerCase().startsWith(searchTerm)) ||
                (this.anywhere && v.toLowerCase().indexOf(searchTerm) >= 0)) {
                /** @type {?} */
                const p = Math.floor(index / this.paginator.rowsOnPage);
                this.paginator.setPage(p + 1);
                break;
            }
            index++;
        }
    }
    /**
     * @private
     * @return {?}
     */
    getSortedData() {
        /** @type {?} */
        let data = this.table.inputData;
        /** @type {?} */
        const sortBy = this.table.sortBy;
        /** @type {?} */
        const sortOrder = this.table.sortOrder === "asc" ? "asc" : "desc";
        if (typeof sortBy === 'string' || sortBy instanceof String) {
            data = orderBy(data, this.caseInsensitiveIteratee((/** @type {?} */ (sortBy))), [sortOrder]);
        }
        else {
            data = orderBy(data, sortBy, [sortOrder]);
        }
        return data;
    }
    /**
     * @private
     * @param {?} sortBy
     * @return {?}
     */
    caseInsensitiveIteratee(sortBy) {
        return (/**
         * @param {?} row
         * @return {?}
         */
        (row) => {
            /** @type {?} */
            let value = row;
            for (const sortByProperty of sortBy.split('.')) {
                if (value) {
                    value = value[sortByProperty];
                }
            }
            if (value && typeof value === 'string' || value instanceof String) {
                return value.toLowerCase();
            }
            return value;
        });
    }
}
TableController.decorators = [
    { type: Component, args: [{
                selector: 'mfTableController',
                template: `
    <div class="extstats-table-controller">
      <mfPaginator #p [mfTable]="table">
        <div class="extstats-pagination" *ngIf="p.dataLength > p.rowsOnPage">
          <div class="help">Page:</div>
          <button type="button" class="extstats-page-item" *ngFor="let page of pages()" [class.active]="p.activePage === page" (click)="p.setPage(page)">
            <a class="extstats-page-link" style="cursor: pointer">{{page}}</a>
          </button>
        </div>

        <div class="filler"></div>

        <div *ngIf="!!searchColumn">
          <div class="help">Search:</div>
          <input type="text" [value]="searchValue" [placeholder]="placeholder" (input)="search($event)">
        </div>

        <div class="filler" *ngIf="!!searchColumn"></div>

        <div class="extstats-pagination" *ngIf="p.dataLength > minRowsOnPage">
          <div class="help">Rows per page:</div>
          <button type="button" class="extstats-page-item" *ngFor="let rows of rowsOnPageSet" [class.active]="p.rowsOnPage===rows" (click)="p.setRowsOnPage(rows)">
            <a class="extstats-page-link" style="cursor: pointer">{{rows}}</a>
          </button>
        </div>
      </mfPaginator>
    </div>`
            }] }
];
TableController.propDecorators = {
    table: [{ type: Input, args: ['table',] }],
    searchColumn: [{ type: Input, args: ['searchColumn',] }],
    placeholder: [{ type: Input, args: ['placeholder',] }],
    anywhere: [{ type: Input, args: ['anywhere',] }],
    paginator: [{ type: ViewChild, args: [Paginator, { static: true },] }]
};
if (false) {
    /** @type {?} */
    TableController.prototype.table;
    /** @type {?} */
    TableController.prototype.searchColumn;
    /** @type {?} */
    TableController.prototype.placeholder;
    /** @type {?} */
    TableController.prototype.anywhere;
    /** @type {?} */
    TableController.prototype.paginator;
    /** @type {?} */
    TableController.prototype.searchValue;
    /** @type {?} */
    TableController.prototype.rowsOnPageSet;
    /** @type {?} */
    TableController.prototype.minRowsOnPage;
}
/**
 * @param {?} page
 * @param {?} totalPages
 * @param {?} pages
 * @return {?}
 */
function addPage(page, totalPages, pages) {
    /** @type {?} */
    const p = Math.round(page);
    if (p > 0 && p <= totalPages && pages.indexOf(p) < 0) {
        pages.push(p);
    }
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/extstats-datatable.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ExtstatsDatatableModule {
}
ExtstatsDatatableModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    BootstrapPaginator,
                    DataTable,
                    DataTableBody,
                    DataTableHead,
                    DefaultSorter,
                    Paginator,
                    TableController
                ],
                imports: [
                    CommonModule
                ],
                exports: [
                    BootstrapPaginator,
                    DataTable,
                    DataTableBody,
                    DataTableHead,
                    DefaultSorter,
                    Paginator,
                    TableController
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * Generated from: lib/model.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function SortEvent() { }
if (false) {
    /** @type {?} */
    SortEvent.prototype.sortBy;
    /** @type {?} */
    SortEvent.prototype.sortOrder;
}
/**
 * @record
 */
function PageEvent() { }
if (false) {
    /** @type {?} */
    PageEvent.prototype.activePage;
    /** @type {?} */
    PageEvent.prototype.rowsOnPage;
    /** @type {?} */
    PageEvent.prototype.dataLength;
}
/**
 * @record
 * @template R
 */
function ColumnParams() { }
if (false) {
    /** @type {?} */
    ColumnParams.prototype.name;
    /** @type {?} */
    ColumnParams.prototype.field;
    /** @type {?|undefined} */
    ColumnParams.prototype.tooltip;
    /** @type {?|undefined} */
    ColumnParams.prototype.classname;
    /** @type {?|undefined} */
    ColumnParams.prototype.valueHtml;
    /** @type {?|undefined} */
    ColumnParams.prototype.valueTooltip;
}
/**
 * @template R
 */
class Column {
    /**
     * @param {?} obj
     */
    constructor(obj) {
        this.name = obj["name"];
        if (obj["field"])
            this.field = obj["field"];
        if (obj["tooltip"])
            this.tooltip = obj["tooltip"];
        if (obj["valueHtml"])
            this.valueHtml = obj["valueHtml"];
        if (obj["valueTooltip"])
            this.valueTooltip = obj["valueTooltip"] || null;
        if (obj["classname"])
            this.classname = obj["classname"];
    }
    /**
     * @param {?} r
     * @return {?}
     */
    valueHtml(r) { return r[this.field] ? r[this.field].toString() : ""; }
    /**
     * @param {?} r
     * @return {?}
     */
    valueTooltip(r) { return null; }
}
if (false) {
    /** @type {?} */
    Column.prototype.name;
    /** @type {?} */
    Column.prototype.field;
    /** @type {?} */
    Column.prototype.tooltip;
    /** @type {?} */
    Column.prototype.classname;
}

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: extstats-datatable.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { BootstrapPaginator, Column, DataTable, DataTableBody, DataTableHead, DefaultSorter, ExtstatsDatatableModule, Paginator, TableController };
//# sourceMappingURL=extstats-datatable.js.map
