import { EventEmitter, Directive, IterableDiffers, Input, Output, Component, Optional, ViewChild, NgModule } from '@angular/core';
import { __values } from 'tslib';
import { includes, orderBy, slice, min } from 'lodash';
import { ReplaySubject } from 'rxjs';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * Generated from: lib/DataTable.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DataTable = /** @class */ (function () {
    function DataTable(differs) {
        this.differs = differs;
        this.inputData = [];
        this.sortBy = "";
        this.sortOrder = "asc";
        this.sortByChange = new EventEmitter();
        this.sortOrderChange = new EventEmitter();
        this.rowsOnPage = 1000;
        this.activePage = 1;
        this.mustRecalculateData = false;
        this.onSortChange = new ReplaySubject(1);
        this.onPageChange = new EventEmitter();
        this.diff = differs.find([]).create(null);
    }
    /**
     * @return {?}
     */
    DataTable.prototype.getSort = /**
     * @return {?}
     */
    function () {
        return { sortBy: this.sortBy, sortOrder: this.sortOrder };
    };
    /**
     * @param {?} sortBy
     * @param {?} sortOrder
     * @return {?}
     */
    DataTable.prototype.setSort = /**
     * @param {?} sortBy
     * @param {?} sortOrder
     * @return {?}
     */
    function (sortBy, sortOrder) {
        if (this.sortBy !== sortBy || this.sortOrder !== sortOrder) {
            this.sortBy = sortBy;
            this.sortOrder = includes(["asc", "desc"], sortOrder) ? sortOrder : "asc";
            this.mustRecalculateData = true;
            this.onSortChange.next({ sortBy: sortBy, sortOrder: sortOrder });
            this.sortByChange.emit(this.sortBy);
            this.sortOrderChange.emit(this.sortOrder);
        }
    };
    /**
     * @return {?}
     */
    DataTable.prototype.getPage = /**
     * @return {?}
     */
    function () {
        return { activePage: this.activePage, rowsOnPage: this.rowsOnPage, dataLength: this.inputData.length };
    };
    /**
     * @param {?} activePage
     * @param {?} rowsOnPage
     * @return {?}
     */
    DataTable.prototype.setPage = /**
     * @param {?} activePage
     * @param {?} rowsOnPage
     * @return {?}
     */
    function (activePage, rowsOnPage) {
        if (this.rowsOnPage !== rowsOnPage || this.activePage !== activePage) {
            this.activePage = this.activePage !== activePage ? activePage : this.calculateNewActivePage(this.rowsOnPage, rowsOnPage);
            this.rowsOnPage = rowsOnPage;
            this.mustRecalculateData = true;
            this.onPageChange.emit({
                activePage: this.activePage,
                rowsOnPage: this.rowsOnPage,
                dataLength: this.inputData ? this.inputData.length : 0
            });
        }
    };
    /**
     * @private
     * @param {?} previousRowsOnPage
     * @param {?} currentRowsOnPage
     * @return {?}
     */
    DataTable.prototype.calculateNewActivePage = /**
     * @private
     * @param {?} previousRowsOnPage
     * @param {?} currentRowsOnPage
     * @return {?}
     */
    function (previousRowsOnPage, currentRowsOnPage) {
        /** @type {?} */
        var firstRowOnPage = (this.activePage - 1) * previousRowsOnPage + 1;
        return Math.ceil(firstRowOnPage / currentRowsOnPage);
    };
    /**
     * @private
     * @return {?}
     */
    DataTable.prototype.recalculatePage = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var lastPage = Math.ceil(this.inputData.length / this.rowsOnPage);
        this.activePage = lastPage < this.activePage ? lastPage : this.activePage;
        this.activePage = this.activePage || 1;
        this.onPageChange.emit({
            activePage: this.activePage,
            rowsOnPage: this.rowsOnPage,
            dataLength: this.inputData.length
        });
    };
    /**
     * @param {?} changes
     * @return {?}
     */
    DataTable.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if (changes["rowsOnPage"]) {
            this.rowsOnPage = changes["rowsOnPage"].previousValue;
            this.setPage(this.activePage, changes["rowsOnPage"].currentValue);
            this.mustRecalculateData = true;
        }
        if (changes["sortBy"] || changes["sortOrder"]) {
            if (!includes(["asc", "desc"], this.sortOrder)) {
                console.warn("angular2-datatable: value for input mfSortOrder must be one of ['asc', 'desc'], but is:", this.sortOrder);
                this.sortOrder = "asc";
            }
            if (this.sortBy) {
                this.onSortChange.next({ sortBy: this.sortBy, sortOrder: this.sortOrder });
            }
            this.mustRecalculateData = true;
        }
        if (changes["inputData"]) {
            this.inputData = changes["inputData"].currentValue || [];
            this.recalculatePage();
            this.mustRecalculateData = true;
        }
    };
    /**
     * @return {?}
     */
    DataTable.prototype.ngDoCheck = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var changes = this.diff.diff(this.inputData);
        if (changes) {
            this.recalculatePage();
            this.mustRecalculateData = true;
        }
        if (this.mustRecalculateData) {
            this.fillData();
            this.mustRecalculateData = false;
        }
    };
    /**
     * @private
     * @return {?}
     */
    DataTable.prototype.fillData = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var offset = (this.activePage - 1) * this.rowsOnPage;
        /** @type {?} */
        var data = this.inputData;
        /** @type {?} */
        var sortBy = this.sortBy;
        if (typeof sortBy === 'string' || sortBy instanceof String) {
            data = orderBy(data, this.caseInsensitiveIteratee((/** @type {?} */ (sortBy))), [this.sortOrder]);
        }
        else {
            data = orderBy(data, sortBy, [this.sortOrder]);
        }
        data = slice(data, offset, offset + this.rowsOnPage);
        this.data = data;
    };
    /**
     * @private
     * @param {?} sortBy
     * @return {?}
     */
    DataTable.prototype.caseInsensitiveIteratee = /**
     * @private
     * @param {?} sortBy
     * @return {?}
     */
    function (sortBy) {
        return (/**
         * @param {?} row
         * @return {?}
         */
        function (row) {
            var e_1, _a;
            /** @type {?} */
            var value = row;
            try {
                for (var _b = __values(sortBy.split('.')), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var sortByProperty = _c.value;
                    if (value) {
                        value = value[sortByProperty];
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
            if (value && typeof value === 'string' || value instanceof String) {
                return value.toLowerCase();
            }
            return value;
        });
    };
    DataTable.decorators = [
        { type: Directive, args: [{
                    selector: 'table[mfData]',
                    exportAs: 'mfDataTable'
                },] }
    ];
    /** @nocollapse */
    DataTable.ctorParameters = function () { return [
        { type: IterableDiffers }
    ]; };
    DataTable.propDecorators = {
        inputData: [{ type: Input, args: ["mfData",] }],
        sortBy: [{ type: Input, args: ["mfSortBy",] }],
        sortOrder: [{ type: Input, args: ["mfSortOrder",] }],
        sortByChange: [{ type: Output, args: ["mfSortByChange",] }],
        sortOrderChange: [{ type: Output, args: ["mfSortOrderChange",] }],
        rowsOnPage: [{ type: Input, args: ["mfRowsOnPage",] }],
        activePage: [{ type: Input, args: ["mfActivePage",] }]
    };
    return DataTable;
}());
if (false) {
    /**
     * @type {?}
     * @private
     */
    DataTable.prototype.diff;
    /** @type {?} */
    DataTable.prototype.inputData;
    /** @type {?} */
    DataTable.prototype.sortBy;
    /** @type {?} */
    DataTable.prototype.sortOrder;
    /** @type {?} */
    DataTable.prototype.sortByChange;
    /** @type {?} */
    DataTable.prototype.sortOrderChange;
    /** @type {?} */
    DataTable.prototype.rowsOnPage;
    /** @type {?} */
    DataTable.prototype.activePage;
    /**
     * @type {?}
     * @private
     */
    DataTable.prototype.mustRecalculateData;
    /** @type {?} */
    DataTable.prototype.data;
    /** @type {?} */
    DataTable.prototype.onSortChange;
    /** @type {?} */
    DataTable.prototype.onPageChange;
    /**
     * @type {?}
     * @private
     */
    DataTable.prototype.differs;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/BootstrapPaginator.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var BootstrapPaginator = /** @class */ (function () {
    function BootstrapPaginator() {
        this.rowsOnPageSet = [];
        this.minRowsOnPage = 0;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    BootstrapPaginator.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if (changes.rowsOnPageSet) {
            this.minRowsOnPage = min(this.rowsOnPageSet);
        }
    };
    BootstrapPaginator.decorators = [
        { type: Component, args: [{
                    selector: "mfBootstrapPaginator",
                    template: "\n    <mfPaginator #p [mfTable]=\"mfTable\">\n        <ul class=\"pagination\" *ngIf=\"p.dataLength > p.rowsOnPage\">\n            <li class=\"page-item\" [class.disabled]=\"p.activePage <= 1\" (click)=\"p.setPage(1)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">&laquo;</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage > 4 && p.activePage + 1 > p.lastPage\" (click)=\"p.setPage(p.activePage - 4)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage-4}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage > 3 && p.activePage + 2 > p.lastPage\" (click)=\"p.setPage(p.activePage - 3)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage-3}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage > 2\" (click)=\"p.setPage(p.activePage - 2)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage-2}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage > 1\" (click)=\"p.setPage(p.activePage - 1)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage-1}}</a>\n            </li>\n            <li class=\"page-item active\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage + 1 <= p.lastPage\" (click)=\"p.setPage(p.activePage + 1)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage+1}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage + 2 <= p.lastPage\" (click)=\"p.setPage(p.activePage + 2)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage+2}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage + 3 <= p.lastPage && p.activePage < 3\" (click)=\"p.setPage(p.activePage + 3)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage+3}}</a>\n            </li>\n            <li class=\"page-item\" *ngIf=\"p.activePage + 4 <= p.lastPage && p.activePage < 2\" (click)=\"p.setPage(p.activePage + 4)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{p.activePage+4}}</a>\n            </li>\n            <li class=\"page-item\" [class.disabled]=\"p.activePage >= p.lastPage\" (click)=\"p.setPage(p.lastPage)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">&raquo;</a>\n            </li>\n        </ul>\n        <ul class=\"pagination pull-right float-sm-right\" *ngIf=\"p.dataLength > minRowsOnPage\">\n            <li class=\"page-item\" *ngFor=\"let rows of rowsOnPageSet\" [class.active]=\"p.rowsOnPage===rows\" (click)=\"p.setRowsOnPage(rows)\">\n                <a class=\"page-link\" style=\"cursor: pointer\">{{rows}}</a>\n            </li>\n        </ul>\n    </mfPaginator>\n    "
                }] }
    ];
    BootstrapPaginator.propDecorators = {
        rowsOnPageSet: [{ type: Input, args: ["rowsOnPageSet",] }],
        mfTable: [{ type: Input, args: ["mfTable",] }]
    };
    return BootstrapPaginator;
}());
if (false) {
    /** @type {?} */
    BootstrapPaginator.prototype.rowsOnPageSet;
    /** @type {?} */
    BootstrapPaginator.prototype.mfTable;
    /** @type {?} */
    BootstrapPaginator.prototype.minRowsOnPage;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/DataTableBody.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template R
 */
var DataTableBody = /** @class */ (function () {
    function DataTableBody(mfTable) {
        this.mfTable = mfTable;
    }
    DataTableBody.decorators = [
        { type: Component, args: [{
                    selector: "[mfBody]",
                    template: "<tr *ngFor=\"let row of mfTable.data\">\n        <td *ngFor=\"let column of columns\"\n            [attr.title]=\"column.valueTooltip(row) || null\"\n            [attr.class]=\"column.classname || null\"\n            [innerHtml]=\"column.valueHtml(row)\"></td>\n    </tr>"
                }] }
    ];
    /** @nocollapse */
    DataTableBody.ctorParameters = function () { return [
        { type: DataTable }
    ]; };
    DataTableBody.propDecorators = {
        columns: [{ type: Input, args: ["columns",] }]
    };
    return DataTableBody;
}());
if (false) {
    /** @type {?} */
    DataTableBody.prototype.columns;
    /** @type {?} */
    DataTableBody.prototype.mfTable;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/DataTableHead.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template R
 */
var DataTableHead = /** @class */ (function () {
    function DataTableHead() {
    }
    DataTableHead.decorators = [
        { type: Component, args: [{
                    selector: "[mfHead]",
                    template: "<tr>\n        <th *ngFor=\"let column of columns\"><mfDefaultSorter by=\"{{column.field}}\" title=\"{{column.tooltip}}\">{{column.name}}</mfDefaultSorter></th>\n    </tr>"
                }] }
    ];
    DataTableHead.propDecorators = {
        columns: [{ type: Input, args: ["columns",] }]
    };
    return DataTableHead;
}());
if (false) {
    /** @type {?} */
    DataTableHead.prototype.columns;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/DefaultSorter.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DefaultSorter = /** @class */ (function () {
    function DefaultSorter(mfTable) {
        this.mfTable = mfTable;
        this.isSortedByMeAsc = false;
        this.isSortedByMeDesc = false;
    }
    /**
     * @return {?}
     */
    DefaultSorter.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.mfTable.onSortChange.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            _this.isSortedByMeAsc = (event.sortBy == _this.sortBy && event.sortOrder == "asc");
            _this.isSortedByMeDesc = (event.sortBy == _this.sortBy && event.sortOrder == "desc");
        }));
    };
    /**
     * @return {?}
     */
    DefaultSorter.prototype.sort = /**
     * @return {?}
     */
    function () {
        if (this.isSortedByMeAsc) {
            this.mfTable.setSort(this.sortBy, "desc");
        }
        else {
            this.mfTable.setSort(this.sortBy, "asc");
        }
    };
    DefaultSorter.decorators = [
        { type: Component, args: [{
                    selector: "mfDefaultSorter",
                    template: "\n        <a style=\"cursor: pointer\" (click)=\"sort()\" class=\"text-nowrap\">\n            <ng-content></ng-content>\n            <span *ngIf=\"isSortedByMeAsc\" class=\"glyphicon glyphicon-triangle-top\" aria-hidden=\"true\"></span>\n            <span *ngIf=\"isSortedByMeDesc\" class=\"glyphicon glyphicon-triangle-bottom\" aria-hidden=\"true\"></span>\n        </a>"
                }] }
    ];
    /** @nocollapse */
    DefaultSorter.ctorParameters = function () { return [
        { type: DataTable }
    ]; };
    DefaultSorter.propDecorators = {
        sortBy: [{ type: Input, args: ["by",] }]
    };
    return DefaultSorter;
}());
if (false) {
    /** @type {?} */
    DefaultSorter.prototype.sortBy;
    /** @type {?} */
    DefaultSorter.prototype.isSortedByMeAsc;
    /** @type {?} */
    DefaultSorter.prototype.isSortedByMeDesc;
    /**
     * @type {?}
     * @private
     */
    DefaultSorter.prototype.mfTable;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/Paginator.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var Paginator = /** @class */ (function () {
    function Paginator(injectMfTable) {
        var _this = this;
        this.injectMfTable = injectMfTable;
        this.dataLength = 0;
        this.onPageChangeSubscriber = (/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            _this.activePage = event.activePage;
            _this.rowsOnPage = event.rowsOnPage;
            _this.dataLength = event.dataLength;
            _this.lastPage = Math.ceil(_this.dataLength / _this.rowsOnPage);
        });
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    Paginator.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        this.mfTable = this.inputMfTable || this.injectMfTable;
        this.onPageChangeSubscriber(this.mfTable.getPage());
        this.mfTable.onPageChange.subscribe(this.onPageChangeSubscriber);
    };
    /**
     * @param {?} pageNumber
     * @return {?}
     */
    Paginator.prototype.setPage = /**
     * @param {?} pageNumber
     * @return {?}
     */
    function (pageNumber) {
        this.mfTable.setPage(pageNumber, this.rowsOnPage);
    };
    /**
     * @param {?} rowsOnPage
     * @return {?}
     */
    Paginator.prototype.setRowsOnPage = /**
     * @param {?} rowsOnPage
     * @return {?}
     */
    function (rowsOnPage) {
        this.mfTable.setPage(this.activePage, rowsOnPage);
    };
    Paginator.decorators = [
        { type: Component, args: [{
                    selector: "mfPaginator",
                    template: "<ng-content></ng-content>"
                }] }
    ];
    /** @nocollapse */
    Paginator.ctorParameters = function () { return [
        { type: DataTable, decorators: [{ type: Optional }] }
    ]; };
    Paginator.propDecorators = {
        inputMfTable: [{ type: Input, args: ["mfTable",] }]
    };
    return Paginator;
}());
if (false) {
    /** @type {?} */
    Paginator.prototype.inputMfTable;
    /**
     * @type {?}
     * @private
     */
    Paginator.prototype.mfTable;
    /** @type {?} */
    Paginator.prototype.activePage;
    /** @type {?} */
    Paginator.prototype.rowsOnPage;
    /** @type {?} */
    Paginator.prototype.dataLength;
    /** @type {?} */
    Paginator.prototype.lastPage;
    /**
     * @type {?}
     * @private
     */
    Paginator.prototype.onPageChangeSubscriber;
    /**
     * @type {?}
     * @private
     */
    Paginator.prototype.injectMfTable;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/TableController.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TableController = /** @class */ (function () {
    function TableController() {
        this.placeholder = 'search term';
        this.anywhere = false;
        this.searchValue = "";
        this.rowsOnPageSet = [20, 30, 50];
        this.minRowsOnPage = 0;
    }
    /**
     * @return {?}
     */
    TableController.prototype.pages = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var result = [];
        /** @type {?} */
        var totalPages = this.paginator.lastPage;
        addPage(1, totalPages, result);
        addPage((this.paginator.activePage + 1) / 2, totalPages, result);
        addPage(this.paginator.activePage - 1, totalPages, result);
        addPage((this.paginator.activePage + totalPages) / 2, totalPages, result);
        addPage(this.paginator.activePage, totalPages, result);
        addPage(this.paginator.activePage + 1, totalPages, result);
        addPage(totalPages, totalPages, result);
        if (result.length < 7) {
            addPage((this.paginator.activePage + totalPages / 10), totalPages, result);
            addPage(this.paginator.activePage - totalPages / 10, totalPages, result);
        }
        result.sort((/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        function (a, b) { return a - b; }));
        return result;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    TableController.prototype.search = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        var e_1, _a;
        /** @type {?} */
        var searchTerm = event.srcElement.value.toLowerCase();
        /** @type {?} */
        var index = 0;
        try {
            for (var _b = __values(this.getSortedData()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var row = _c.value;
                /** @type {?} */
                var v = row[this.searchColumn].toString();
                if ((!this.anywhere && v.toLowerCase().startsWith(searchTerm)) ||
                    (this.anywhere && v.toLowerCase().indexOf(searchTerm) >= 0)) {
                    /** @type {?} */
                    var p = Math.floor(index / this.paginator.rowsOnPage);
                    this.paginator.setPage(p + 1);
                    break;
                }
                index++;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    /**
     * @private
     * @return {?}
     */
    TableController.prototype.getSortedData = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var data = this.table.inputData;
        /** @type {?} */
        var sortBy = this.table.sortBy;
        /** @type {?} */
        var sortOrder = this.table.sortOrder === "asc" ? "asc" : "desc";
        if (typeof sortBy === 'string' || sortBy instanceof String) {
            data = orderBy(data, this.caseInsensitiveIteratee((/** @type {?} */ (sortBy))), [sortOrder]);
        }
        else {
            data = orderBy(data, sortBy, [sortOrder]);
        }
        return data;
    };
    /**
     * @private
     * @param {?} sortBy
     * @return {?}
     */
    TableController.prototype.caseInsensitiveIteratee = /**
     * @private
     * @param {?} sortBy
     * @return {?}
     */
    function (sortBy) {
        return (/**
         * @param {?} row
         * @return {?}
         */
        function (row) {
            var e_2, _a;
            /** @type {?} */
            var value = row;
            try {
                for (var _b = __values(sortBy.split('.')), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var sortByProperty = _c.value;
                    if (value) {
                        value = value[sortByProperty];
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_2) throw e_2.error; }
            }
            if (value && typeof value === 'string' || value instanceof String) {
                return value.toLowerCase();
            }
            return value;
        });
    };
    TableController.decorators = [
        { type: Component, args: [{
                    selector: 'mfTableController',
                    template: "\n    <div class=\"extstats-table-controller\">\n      <mfPaginator #p [mfTable]=\"table\">\n        <div class=\"extstats-pagination\" *ngIf=\"p.dataLength > p.rowsOnPage\">\n          <div class=\"help\">Page:</div>\n          <button type=\"button\" class=\"extstats-page-item\" *ngFor=\"let page of pages()\" [class.active]=\"p.activePage === page\" (click)=\"p.setPage(page)\">\n            <a class=\"extstats-page-link\" style=\"cursor: pointer\">{{page}}</a>\n          </button>\n        </div>\n\n        <div class=\"filler\"></div>\n\n        <div *ngIf=\"!!searchColumn\">\n          <div class=\"help\">Search:</div>\n          <input type=\"text\" [value]=\"searchValue\" [placeholder]=\"placeholder\" (input)=\"search($event)\">\n        </div>\n\n        <div class=\"filler\" *ngIf=\"!!searchColumn\"></div>\n\n        <div class=\"extstats-pagination\" *ngIf=\"p.dataLength > minRowsOnPage\">\n          <div class=\"help\">Rows per page:</div>\n          <button type=\"button\" class=\"extstats-page-item\" *ngFor=\"let rows of rowsOnPageSet\" [class.active]=\"p.rowsOnPage===rows\" (click)=\"p.setRowsOnPage(rows)\">\n            <a class=\"extstats-page-link\" style=\"cursor: pointer\">{{rows}}</a>\n          </button>\n        </div>\n      </mfPaginator>\n    </div>"
                }] }
    ];
    TableController.propDecorators = {
        table: [{ type: Input, args: ['table',] }],
        searchColumn: [{ type: Input, args: ['searchColumn',] }],
        placeholder: [{ type: Input, args: ['placeholder',] }],
        anywhere: [{ type: Input, args: ['anywhere',] }],
        paginator: [{ type: ViewChild, args: [Paginator, { static: true },] }]
    };
    return TableController;
}());
if (false) {
    /** @type {?} */
    TableController.prototype.table;
    /** @type {?} */
    TableController.prototype.searchColumn;
    /** @type {?} */
    TableController.prototype.placeholder;
    /** @type {?} */
    TableController.prototype.anywhere;
    /** @type {?} */
    TableController.prototype.paginator;
    /** @type {?} */
    TableController.prototype.searchValue;
    /** @type {?} */
    TableController.prototype.rowsOnPageSet;
    /** @type {?} */
    TableController.prototype.minRowsOnPage;
}
/**
 * @param {?} page
 * @param {?} totalPages
 * @param {?} pages
 * @return {?}
 */
function addPage(page, totalPages, pages) {
    /** @type {?} */
    var p = Math.round(page);
    if (p > 0 && p <= totalPages && pages.indexOf(p) < 0) {
        pages.push(p);
    }
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/extstats-datatable.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ExtstatsDatatableModule = /** @class */ (function () {
    function ExtstatsDatatableModule() {
    }
    ExtstatsDatatableModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [
                        BootstrapPaginator,
                        DataTable,
                        DataTableBody,
                        DataTableHead,
                        DefaultSorter,
                        Paginator,
                        TableController
                    ],
                    imports: [
                        CommonModule
                    ],
                    exports: [
                        BootstrapPaginator,
                        DataTable,
                        DataTableBody,
                        DataTableHead,
                        DefaultSorter,
                        Paginator,
                        TableController
                    ]
                },] }
    ];
    return ExtstatsDatatableModule;
}());

/**
 * @fileoverview added by tsickle
 * Generated from: lib/model.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function SortEvent() { }
if (false) {
    /** @type {?} */
    SortEvent.prototype.sortBy;
    /** @type {?} */
    SortEvent.prototype.sortOrder;
}
/**
 * @record
 */
function PageEvent() { }
if (false) {
    /** @type {?} */
    PageEvent.prototype.activePage;
    /** @type {?} */
    PageEvent.prototype.rowsOnPage;
    /** @type {?} */
    PageEvent.prototype.dataLength;
}
/**
 * @record
 * @template R
 */
function ColumnParams() { }
if (false) {
    /** @type {?} */
    ColumnParams.prototype.name;
    /** @type {?} */
    ColumnParams.prototype.field;
    /** @type {?|undefined} */
    ColumnParams.prototype.tooltip;
    /** @type {?|undefined} */
    ColumnParams.prototype.classname;
    /** @type {?|undefined} */
    ColumnParams.prototype.valueHtml;
    /** @type {?|undefined} */
    ColumnParams.prototype.valueTooltip;
}
/**
 * @template R
 */
var  /**
 * @template R
 */
Column = /** @class */ (function () {
    function Column(obj) {
        this.name = obj["name"];
        if (obj["field"])
            this.field = obj["field"];
        if (obj["tooltip"])
            this.tooltip = obj["tooltip"];
        if (obj["valueHtml"])
            this.valueHtml = obj["valueHtml"];
        if (obj["valueTooltip"])
            this.valueTooltip = obj["valueTooltip"] || null;
        if (obj["classname"])
            this.classname = obj["classname"];
    }
    /**
     * @param {?} r
     * @return {?}
     */
    Column.prototype.valueHtml = /**
     * @param {?} r
     * @return {?}
     */
    function (r) { return r[this.field] ? r[this.field].toString() : ""; };
    /**
     * @param {?} r
     * @return {?}
     */
    Column.prototype.valueTooltip = /**
     * @param {?} r
     * @return {?}
     */
    function (r) { return null; };
    return Column;
}());
if (false) {
    /** @type {?} */
    Column.prototype.name;
    /** @type {?} */
    Column.prototype.field;
    /** @type {?} */
    Column.prototype.tooltip;
    /** @type {?} */
    Column.prototype.classname;
}

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: extstats-datatable.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { BootstrapPaginator, Column, DataTable, DataTableBody, DataTableHead, DefaultSorter, ExtstatsDatatableModule, Paginator, TableController };
//# sourceMappingURL=extstats-datatable.js.map
